function [rxcca lag] = vsXCCA(X,Y,lag,showProcess)
% vsXCCA - computes the cross canonical correlation between time series
% X and Y
%
%
%
% Syntax:  [rxcca delay] = vsXCCA(X,Y,lag,showProcess)
%
% Inputs:
%    X, Y           - N-by-P1 and N-by-P2 data matrices X and Y.  
%    X and Y must have the same number of observations (rows) but can 
%    have different numbers of variables (cols).
%    lag            - possible time delay range between X and Y
%
% Outputs:
%    rxcca  - the cross canonical correlation function/coefficients
%    lag - time delay range between X and Y
%
% Examples:
% rxcca = vsXCCA(synData_1',synData_2',lag,showProcess);
% 
% [~, cMax] = max(rxcca);
% delay = length(lag) - cMax;
%
% Author: Chen Change Loy
%
%---------------------------- BEGIN CODE ----------------------------------

if nargin < 4
    showProcess = 0;
end

noFeatureX = size(X,2);
noFeatureY = size(Y,2);

% Add padded zero to X
zeroLength = length(lag) - 1;
zero_padX = zeros([zeroLength noFeatureX]);
X_padded = [zero_padX; X; zero_padX];
X_padded_orig = X_padded; % save an original copy


% Initialisation
rBest = 0;
totalStep =  2 *length(lag) - 1;
rxcca = zeros(totalStep,1);

for step = 1:totalStep
    X_padded = X_padded_orig;
    
    zero_padY_start = zeros([step-1 noFeatureY]);
    zero_padY_end = zeros([totalStep-step noFeatureY]);
    Y_padded = [zero_padY_start; Y; zero_padY_end]; 

    
    [~,~,r,~,~,~] = canoncorr(X_padded,Y_padded);
    rxcca(step) = mean(r);
    
    if showProcess
        subplot(3,1,1);
        plot(X_padded);
        subplot(3,1,2);
        plot(Y_padded);
        subplot(3,1,3);
        plot(rxcca);        
        pause;
    end
    
    if mean(r) > mean(rBest) % previous r > rBest
        rBest = r;
    end
end

%---------------------------- END OF CODE ---------------------------------


